#include "IZ_input.h"

void IZ_HandleSDLInputEvents(SDL_Event e, IZ_InputState* state) {
	IZ_HandleJoystickEvents(e, &state->joystick_state, &state->action);
	IZ_HandleKeyboardEvents(e, &state->keyboard_state, &state->action);
}

void IZ_HandlePortMIDIInputEvents(PmEvent e, IZ_InputState* state) {
	IZ_HandleMIDIInputEvents(e, &state->midi_input_state, &state->action);
}

void IZ_InitializeInput(const char* config_path, IZ_InputState* state) {
	*state = (IZ_InputState) {};

	IZ_ProcedureResult joystick_result = IZ_InitializeJoystickState(config_path, &state->joystick_state);
	if (joystick_result) {
		fprintf_s(stderr, "Error committing joystick config. Code: %u.\n", joystick_result);
	}

	IZ_ProcedureResult keyboard_result = IZ_InitializeKeyboardState(config_path, &state->keyboard_state);
	if (keyboard_result) {
		fprintf_s(stderr, "Error committing keyboard config. Code: %u.\n", keyboard_result);
	}

	IZ_ProcedureResult midi_input_result = IZ_InitializeMIDIInput(config_path, &state->midi_input_state);
	if (midi_input_result) {
		fprintf_s(stderr, "Error committing MIDI input config. Code: %u.\n", midi_input_result);
	}

	for (uint8_t p = 0; p < PLAYERS; p += 1) {
		state->action[p] = 0;
	}
}

void IZ_TeardownInput(IZ_InputState* state) {
	IZ_TeardownJoystickState(&state->joystick_state);
	IZ_TeardownMIDIInput(&state->midi_input_state);
}
