#ifndef IZ_JOYSTICK_H
#define IZ_JOYSTICK_H

#include <SDL_joystick.h>
#include <SDL_events.h>
#include <minIni.h>
#include "IZ_action.h"

typedef uint8_t IZ_PadButton;

static const uint16_t IZ_DEFAULT_AXIS_THRESHOLD = 8000;

typedef enum {
	IZ_JOY_AXIS_DIRECTION_HORIZONTAL1 = 0,
	IZ_JOY_AXIS_DIRECTION_VERTICAL1 = 1,
	IZ_JOY_AXIS_DIRECTION_HORIZONTAL2 = 3,
	IZ_JOY_AXIS_DIRECTION_VERTICAL2 = 4,
} IZ_JoyAxisDirection;

typedef struct {
	uint16_t axis_threshold;
	SDL_JoystickID device_id;
	IZ_PadButton control_mapping[CONTROLS];
} IZ_JoystickConfig;

typedef struct {
	SDL_Joystick* device;
	IZ_JoystickConfig config;
} IZ_JoystickState;

static IZ_PadButton IZ_DEFAULT_JOYSTICK_CONTROLS[PLAYERS][CONTROLS] = {
	{
		255,
		255,
		255,
		255,
		11,
		10,
		1,
		0,
		4,
		3,
		6,
		7,
		8,
		9,
		13,
		14,
	},
};

IZ_ProcedureResult IZ_SaveJoystickConfig(const char*, IZ_JoystickState(*)[PLAYERS]);

void IZ_HandleJoystickEvents(SDL_Event, IZ_JoystickState(*)[PLAYERS], IZ_Action(*)[PLAYERS]);

IZ_ProcedureResult IZ_InitializeJoystickState(const char*, IZ_JoystickState(*)[PLAYERS]);

void IZ_TeardownJoystickState(IZ_JoystickState(*)[PLAYERS]);

#endif
