#include "IZ_video.h"

IZ_ProcedureResult IZ_SaveVideoConfig(const char* config_path, IZ_VideoConfig* config) {
	if (!ini_putl("Video", "Width", config->width, config_path)) {
		return 1;
	}
	if (!ini_putl("Video", "Height", config->height, config_path)) {
		return 1;
	}
	if (!ini_putl("Video", "MaxFps", config->max_fps, config_path)) {
		return 1;
	}

	return 0;
}

void IZ_LoadVideoConfig(const char* config_path, IZ_VideoConfig* config) {
	config->width = ini_getl("Video", "Width", 640l, config_path);
	config->height = ini_getl("Video", "Height", 480l, config_path);
	config->max_fps = ini_getl("Video", "MaxFps", 30, config_path);
}

IZ_ProcedureResult IZ_InitializeVideo(const char* config_path, IZ_VideoState* state) {
	*state = (IZ_VideoState) {};

	IZ_LoadVideoConfig(config_path, &state->config);
	if (IZ_SaveVideoConfig(config_path, &state->config)) {
		fprintf_s(stderr, "Error committing video config.\n");
	}
	state->last_update_at = 0u;

	SDL_Window* window = SDL_CreateWindow(
		APP_NAME,
		SDL_WINDOWPOS_CENTERED,
		SDL_WINDOWPOS_CENTERED,
		state->config.width,
		state->config.height,
		SDL_WINDOW_SHOWN
	);
	if (window == NULL) {
		fprintf_s(stderr, "Window could not be created! SDL_Error: %s\n", SDL_GetError());
		return 1;
	}
	state->window = window;
	state->surface = SDL_GetWindowSurface(window);

	return 0;
}

void IZ_UpdateVideo(IZ_VideoState* video_state, IZ_InputState* input_states, uint64_t ticks) {
	if (ticks - video_state->last_update_at > 1000 / video_state->config.max_fps) {
		// Update window
		SDL_FillRect(video_state->surface, NULL, SDL_MapRGB(video_state->surface->format, 0x00, 0x00, 0x00));
		for (uint8_t i = 0; i < 64; i += 1) {
			const uint8_t column = (64 - i) % 32;
			const uint8_t row = i / 32;
			const uint64_t bitflag = (0x1lu << i);
			const uint8_t size = 4;
			if (ticks & bitflag) {
				SDL_FillRect(video_state->surface, &(SDL_Rect) {
					column * size,
					video_state->config.height - ((row + 1) * size),
					size,
					size
				}, SDL_MapRGB(video_state->surface->format, 0x00, 0xff, 0xff));
			}
		}

		// TODO refer to app's state
		for (uint8_t p = 0; p < PLAYERS; p += 1) {
			for (uint8_t i = 0; i < CONTROLS; i += 1) {
				const uint8_t column = (i % 4) + (p * 4);
				const uint8_t row = i / 4;
				const IZ_Action bitflag = (0x1 << i);
				const uint8_t size = 4;
				if (input_states->action[p] & bitflag) {
					SDL_FillRect(video_state->surface, &(SDL_Rect) {
						column * size,
						row * size,
						size,
						size
					}, SDL_MapRGB(video_state->surface->format, 0xff, 0xff, 0x00));
				}
			}
		}
		SDL_UpdateWindowSurface(video_state->window);
		video_state->last_update_at = ticks;
	}
}

void IZ_TeardownVideo(IZ_VideoState* state) {
	SDL_DestroyWindow(state->window);
}
